# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
# Author:  Jess Corrius <jcorrius@fluendo.com>

from elisa.core import common
from elisa.core.media_uri import MediaUri

from elisa.plugins.poblesec.actions import Action
from elisa.plugins.poblesec.base.hierarchy import HierarchyController
from elisa.plugins.poblesec.base.list import GenericListViewMode

from elisa.plugins.base.models.media import PlayableModel

from twisted.internet import defer

import os

class ThumbnailMixin(object):

    def __init__(self):
        # FIXME: we need the frontend to get a reference to the gst_metadata
        # instance. This a cheap - UGLY - way to get the frontend without
        # changing a lot of client code. It is really ugly as we assume there
        # is only one frontend, which might not be the case in the future...
        frontend = common.application.interface_controller.frontends.values()[0]
        # Retrieve and store a reference to gst_metadata
        controllers = frontend.retrieve_controllers('/poblesec')
        self.gst_metadata = controllers[0].gst_metadata

    def _updated_thumbnail(self, thumbnail, item):
        item.thumbnail_uri = unicode(thumbnail)
        return thumbnail

    def _request_thumbnail(self, path):
        def got_metadata(metadata):
            return metadata['thumbnail']
        
        metadata = {'uri': MediaUri("file://%s" % path), 'thumbnail': None,}
        
        dfr = self.gst_metadata.get_metadata(metadata)
        dfr.addCallback(got_metadata)
        return dfr

class VideoViewMode(GenericListViewMode, ThumbnailMixin):
    
    def get_label(self, item):
        return defer.succeed(os.path.basename(item.file_path))

    def get_default_image(self, item):
        resource = 'elisa.plugins.poblesec.file_video'
        return resource
    
    def get_image(self, item, theme):
        if hasattr(item, 'thumbnail_uri') and item.thumbnail_uri != None:
            return defer.succeed(item.thumbnail_uri)
        
        dfr = self._request_thumbnail(item.file_path)
        dfr.addCallback(self._updated_thumbnail, item)
        return dfr
    
    def get_preview_image(self, item, theme):
        if hasattr(item, 'thumbnail_uri') and item.thumbnail_uri != None:
            return item.thumbnail_uri
        else:
            return None

class VideoController(HierarchyController):
    
    def node_clicked(self, widget, item):
        widget_index = self.nodes._widget_index_from_item_index(widget.model.index(item))
        selected_item = self.nodes._widgets[widget_index]

        if selected_item != self._previous_clicked:
            selected_item.activate(previous=self._previous_clicked)

            self._previous_clicked = selected_item

            if isinstance(item, Action):
                dfr = item.run()
            
            controllers = self.frontend.retrieve_controllers('/poblesec/video_player')
            player_controller = controllers[0]

            # enqueue and play the clicked file
            playable_model = PlayableModel()
            playable_model.uri = MediaUri("file://%s" % item.file_path) 
            player_controller.player.play_model(playable_model)
            
            controllers = self.frontend.retrieve_controllers('/poblesec')
            main = controllers[0]
            main.show_video_player()
            self.stop_loading_animation()

    